@@ -1,8 +1,8 @@
 // enigmailCommon.js: shared JS functions for Enigmail
 
 // This Enigmail version and compatible Enigmime version
-var gEnigmailVersion = "0.73.1.0";
-var gEnigmimeVersion = "0.73.1.0";
+var gEnigmailVersion = "0.76.2.0";
+var gEnigmimeVersion = "0.76.2.0";
 
 // Maximum size of message directly processed by Enigmail
 const ENIG_MSG_BUFFER_SIZE = 96000;
@@ -18,6 +18,15 @@ const ENIG_ENIGMAIL_CONTRACTID    = "@mozdev.org/enigmail/enigmail;1";
 const ENIG_ENIGMIMELISTENER_CONTRACTID = "@mozilla.org/enigmail/mime-listener;1";
 const ENIG_ENIGMIMESERVICE_CONTRACTID = "@mozdev.org/enigmail/enigmimeservice;1";
 const ENIG_STRINGBUNDLE_CONTRACTID = "@mozilla.org/intl/stringbundle;1";
+const ENIG_LOCAL_FILE_CONTRACTID = "@mozilla.org/file/local;1";
+const ENIG_DIRSERVICE_CONTRACTID = "@mozilla.org/file/directory_service;1";
+const ENIG_MIME_CONTRACTID = "@mozilla.org/mime;1";
+const ENIG_WMEDIATOR_CONTRACTID = "@mozilla.org/rdf/datasource;1?name=window-mediator";
+const ENIG_APPSHELL_MEDIATOR_CONTRACTID = "@mozilla.org/appshell/window-mediator;1"
+
+const ENIG_LOCALFILEOUTPUTSTREAM_CONTRACTID =
+                              "@mozilla.org/network/file-output-stream;1";
+const ENIG_STANDARD_URL_CONTRACTID = "@mozilla.org/network/standard-url;1";
 
 const ENIG_STREAMCONVERTERSERVICE_CID_STR =
       "{892FFEB0-3F80-11d3-A16C-0050041CAF44}";
@@ -44,6 +53,9 @@ const PGP_MIME_NEVER    = 0;
 const PGP_MIME_POSSIBLE = 1;
 const PGP_MIME_ALWAYS   = 2;
 
+// property name for temporary directory service
+const ENIG_TEMPDIR_PROP = "TmpD";
+
 var gUsePGPMimeOptionList = ["usePGPMimeNever", "usePGPMimePossible",
                              "usePGPMimeAlways"];
 
@@ -51,11 +63,18 @@ var gEnigDefaultEncryptionOptions = ["defaultEncryptionNone",
                                      "defaultEncryptionOnly",
                                      "defaultEncryptionSign"];
 
+var gEnigRecipientsSelectionOptions = ["askRecipientsNever",
+                                       "askRecipientsClever",
+                                       "askRecipientsAlways"];
+
+var gEnigImmediateSendOptions = ["sendLater",
+                                 "sendNow"];
+
 const ENIG_BUTTON_POS_0           = 1;
 const ENIG_BUTTON_POS_1           = 1 << 8;
 const ENIG_BUTTON_POS_2           = 1 << 16;
 const ENIG_BUTTON_TITLE_IS_STRING = 127;
-  
+
 const ENIG_THREE_BUTTON_STRINGS   = (ENIG_BUTTON_TITLE_IS_STRING * ENIG_BUTTON_POS_0) +
                                (ENIG_BUTTON_TITLE_IS_STRING * ENIG_BUTTON_POS_1) +
                                (ENIG_BUTTON_TITLE_IS_STRING * ENIG_BUTTON_POS_2);
@@ -81,13 +100,20 @@ var gEnigmailPrefDefaults = {"configuredVersion":"",
                              "confirmBeforeSend":false,
                              "doubleDashSeparator":false,
                              "maxIdleMinutes":5,
-                             "keyserver":"www.keyserver.net",
+                             "keyserver":"wwwkeys.pgp.net",
                              "autoDecrypt":true,
                              "captureWebMail":false,
                              "useMimeExperimental":false,
                              "disableSMIMEui":true,
                              "parseAllHeaders":true,
-                             "show_headers":1
+                             "show_headers":1,
+                             "hushMailSupport":false,
+                             "encryptAttachments":1,
+                             "inlineAttachAsciiArmor":false,
+                             "inlineAttachExt":".pgp",
+                             "handleDoubleClick":false,
+                             "sendImmediately":true,
+                             "recipientsSelectionOption":1
                             };
 
 var gEnigLogLevel = 2;     // Output only errors/warnings by default
@@ -205,6 +231,11 @@ function GetEnigmailSvc() {
 
     DEBUG_LOG("enigmailCommon.js: GetEnigmailSvc: "+configuredVersion+"\n");
 
+    if (firstInitialization && gEnigmailSvc.initialized &&
+        gEnigmailSvc.agentType && gEnigmailSvc.agentType == "pgp") {
+      EnigAlert(EnigGetString("pgpNotSupported"));
+    }
+
     if (gEnigmailSvc.initialized && (gEnigmailVersion != configuredVersion)) {
       EnigConfigure();
     }
@@ -337,10 +368,6 @@ function EnigConfigure() {
 ///////////////////////////////////////////////////////////////////////////////
 // File read/write operations
 
-const ENIG_LOCAL_FILE_CONTRACTID = "@mozilla.org/file/local;1";
-
-const ENIG_LOCALFILEOUTPUTSTREAM_CONTRACTID =
-                              "@mozilla.org/network/file-output-stream;1";
 
 const ENIG_RDONLY      = 0x01;
 const ENIG_WRONLY      = 0x02;
@@ -348,8 +375,6 @@ const ENIG_CREATE_FILE = 0x08;
 const ENIG_TRUNCATE    = 0x20;
 const ENIG_DEFAULT_FILE_PERMS = 0600;
 
-const EnigFileChannel = new Components.Constructor( "@mozilla.org/network/local-file-channel;1", "nsIFileChannel" );
-
 const EnigInputStream = new Components.Constructor( "@mozilla.org/scriptableinputstream;1", "nsIScriptableInputStream" );
 
 function EnigCreateFileStream(filePath, permissions) {
@@ -409,31 +434,45 @@ function EnigWriteFileContents(filePath, data, permissions) {
   return true;
 }
 
-// maxBytes == -1 => read whole file
-function EnigReadFileContents(localFile, maxBytes) {
-  DEBUG_LOG("enigmailCommon.js: EnigReadFileContents: file="+localFile.leafName+
+// maxBytes == -1 => read everything
+function EnigReadURLContents(url, maxBytes) {
+  DEBUG_LOG("enigmailCommon.js: EnigReadURLContents: url="+url+
             ", "+maxBytes+"\n");
 
-  var fileChannel = new EnigFileChannel();
+  var ioServ = Components.classes[ENIG_IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
+  if (!ioServ)
+    throw Components.results.NS_ERROR_FAILURE;
+
+  var fileChannel = ioServ.newChannel(url, null, null)
+
+  var rawInStream = fileChannel.open();
+
+  var scriptableInStream = new EnigInputStream();
+  scriptableInStream.init(rawInStream);
+
+  var available = scriptableInStream.available()
+  if ((maxBytes < 0) || (maxBytes > available))
+    maxBytes = available;
+
+  var urlContents = scriptableInStream.read(maxBytes);
+
+  scriptableInStream.close();
+
+  return urlContents;
+}
+
+// maxBytes == -1 => read whole file
+function EnigReadFileContents(localFile, maxBytes) {
+
+  DEBUG_LOG("enigmailCommon.js: EnigReadFileContents: file="+localFile.leafName+
+            ", "+maxBytes+"\n");
 
   if (!localFile.exists() || !localFile.isReadable())
     throw Components.results.NS_ERROR_FAILURE;
 
-  fileChannel.init(localFile, ENIG_RDONLY, 0);
 
-  var rawInStream = fileChannel.open();
+  return EnigReadURLContents("file://"+localFile.path, maxBytes);
 
-  var scriptableInStream = new EnigInputStream();    
-  scriptableInStream.init(rawInStream);
-
-  if ((maxBytes < 0) || (maxBytes > localFile.fileSize))
-    maxBytes = localFile.fileSize;
-
-  var fileContents = scriptableInStream.read(maxBytes);
-
-  scriptableInStream.close();
-
-  return fileContents;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -765,6 +804,7 @@ function EnigGetDeepText(node, findStr) {
 
   DEBUG_LOG("enigmailCommon.js: EnigDeepText: <" + node.tagName + ">, '"+findStr+"'\n");
 
+  /*
   try {
     var enigMimeService = Components.classes[ENIG_ENIGMIMESERVICE_CONTRACTID].getService(Components.interfaces.nsIEnigMimeService);
 
@@ -774,36 +814,70 @@ function EnigGetDeepText(node, findStr) {
   } catch (ex) {
     DEBUG_LOG("enigmailCommon.js: EnigDeepText: Failed to access EnigMimeService\n");
   }
-
-  var depth = 0;
-  var textArr = [""];
-
-  while (node) {
-
-    while (node.hasChildNodes()) {
-       depth++;
-       node = node.firstChild;
-    }
-
-    if (node.nodeType == Node.TEXT_NODE) {
-      textArr.push(node.data);
-    }
-
-    while (!node.nextSibling && (depth > 0)) {
-      depth--;
-      node = node.parentNode;
-    }
-
-    if (depth > 0) {
-      node = node.nextSibling;
-    } else {
-      node = null;
+  */
+  
+  if (findStr) {
+    if (node.innerHTML.replace(/&nbsp;/g, " ").indexOf(findStr) < 0) {
+      // exit immediately if findStr is not found at all
+      return "";
     }
   }
+  
+  // EnigDumpHTML(node);
+  
+  var plainText = EnigParseChildNodes(node);
+  // Replace non-breaking spaces with plain spaces
+  plainText = plainText.replace(/\xA0/g," ");
+  
+  if (findStr) {
+     if (plainText.indexOf(findStr) < 0) {
+        return "";
+     }
+  }
+  
+
+  return plainText;
 
-  return textArr.join("");
 }
 
+// extract the plain text by iterating recursively through all nodes
+function EnigParseChildNodes(node) {
+
+  var plainText="";
+  
+  if (node.nodeType == Node.TEXT_NODE) {
+    // text node
+    plainText = plainText.concat(node.data);
+  }
+  else {
+  
+    if (node.nodeType == Node.ELEMENT_NODE) {
+      if (node.tagName=="IMG" && node.className=="moz-txt-smily") {
+        // get the "alt" part of graphical smileys to ensure correct 
+        // verification of signed messages
+        if (node.getAttribute("alt")) {
+            plainText = plainText.concat(node.getAttribute("alt"));
+        }
+      }
+    }
+
+    var child = node.firstChild;
+    // iterate over child nodes
+    while (child) {
+      if (! (child.nodeType == Node.ELEMENT_NODE &&
+            child.tagName == "BR" && 
+            ! child.hasChildNodes())) {
+        // optimization: don't do an extra loop for the very frequent <BR> elements
+        plainText = plainText.concat(EnigParseChildNodes(child));
+      }
+      child = child.nextSibling;
+    }
+  }
+  
+  return plainText;
+}
+
+
 // Dump HTML content as plain text
 function EnigDumpHTML(node)
 {
@@ -857,9 +931,6 @@ function EnigDumpHTML(node)
 /////////////////////////
 
 
-const ENIG_WMEDIATOR_CONTRACTID = "@mozilla.org/rdf/datasource;1?name=window-mediator";
-
-
 function EnigClearPassphrase() {
   DEBUG_LOG("enigmailCommon.js: EnigClearPassphrase: \n");
 
@@ -939,7 +1010,15 @@ function EnigLoadURLInNavigatorWindow(url, aOpenFlag)
 
   // if not, get the most recently used browser window
   if (!navWindow) {
-    var wm = Components.classes[ENIG_WMEDIATOR_CONTRACTID].getService(Components.interfaces.nsIWindowMediator);
+    var wm;
+    try {
+      // Mozilla up to 1.0
+      wm = Components.classes[ENIG_WMEDIATOR_CONTRACTID].getService(Components.interfaces.nsIWindowMediator);
+    }
+    catch (ex) {
+      // Mozilla 1.1 and newer
+      wm = Components.classes[ENIG_APPSHELL_MEDIATOR_CONTRACTID].getService(Components.interfaces.nsIWindowMediator);
+    }
     navWindow = wm.getMostRecentWindow("navigator:browser");
   }
 
@@ -990,3 +1069,52 @@ function EnigGetString(aStr) {
   }
   return null;
 }
+
+// Remove all quoted strings (and angle brackets) from a list of email
+// addresses, returning a list of pure email addresses
+function enigStripEmail(mailAddrs) {
+
+  var qStart, qEnd;
+  while ((qStart = mailAddrs.indexOf('"')) != -1) {
+     qEnd = mailAddrs.indexOf('"', qStart+1);
+     if (qEnd == -1) {
+       ERROR_LOG("enigmailMsgComposeOverlay.js: enigStripEmail: Unmatched quote in mail address: "+mailAddrs+"\n");
+       throw Components.results.NS_ERROR_FAILURE;
+     }
+  
+     mailAddrs = mailAddrs.substring(0,qStart) + mailAddrs.substring(qEnd+1);
+  }
+  
+  // Eliminate all whitespace, just to be safe
+  mailAddrs = mailAddrs.replace(/\s+/g,"");
+  
+  // Extract pure e-mail address list (stripping out angle brackets)
+  mailAddrs = mailAddrs.replace(/(^|,)[^,]*<([^>]+)>[^,]*/g,"$1$2");
+
+  return mailAddrs;
+}
+
+
+//get path for temporary directory (e.g. /tmp, C:\TEMP)
+function EnigGetTempDir() {
+  var tmpDir;
+
+  try {
+    var ds = Components.classes[ENIG_DIRSERVICE_CONTRACTID].getService();
+    var dsprops = ds.QueryInterface(Components.interfaces.nsIProperties);
+    var tmpDirComp = dsprops.get(ENIG_TEMPDIR_PROP, Components.interfaces.nsILocalFile);
+    tmpDir=tmpDirComp.path;
+  }
+  catch (ex) {
+    // let's guess ...
+    var httpHandler = ioServ.getProtocolHandler("http");
+    httpHandler = httpHandler.QueryInterface(Components.interfaces.nsIHttpProtocolHandler);
+    isWin = (httpHandler.platform.search(/Win/i) == 0);
+    if (isWin) {
+      tmpDir="C:\\TEMP";
+    } else {
+      tmpDir="/tmp";
+    }
+  }
+  return tmpDir;
+}
